
------------------------------------
--CIM - Custom Icon Maker for 3ds max 4 
--Version 0.55
--by Borislav Petrov [Boboland]
--bobo@email.archlab.tuwien.ac.at
--http://gfxcentral.com/bobo/
--Started:	01/03/99
--Edited: 	01/31/01
------------------------------------

utility custom_icon_maker "Custom Icon Maker"
(
--------------------------------------------------------------------
--FUNCTION TO DISPLAY AN ICON FROM LIBRARY--------------------------
--------------------------------------------------------------------

fn display_icon_number o_b l_w d_b i_n =
(
	copy o_b d_b
	redline = #()
	for i = 1 to 24 do append redline ([255,0,0] as color)
	setpixels d_b [(i_n-1)*24,0] redline
	setpixels d_b [(i_n-1)*24,23] redline
	redline = #()
	append redline ([255,0,0] as color)
	for i = 1 to 22 do
	(
		setpixels d_b [(i_n-1)*24,i] redline
		setpixels d_b [(23+((i_n-1)*24)),i] redline
	)--end i loop
	d_b
)--end fn


--------------------------------------------------------------------
--INIT VARIABLES AND ARRAYS-----------------------------------------
--------------------------------------------------------------------

local lib_list = #()
local lib_database = #()
local void_bitmap16 = bitmap 16 15
local void_bitmap24 = bitmap 24 24
local void_bitmap120 = bitmap 120 120
local dlib_bmp = bitmap 24 24
local olib_bmp = bitmap 24 24
local render_done_flag = false

--------------------------------------------------------------------
--COMMAND PANEL UI -------------------------------------------------
--------------------------------------------------------------------

group "About..."
(
	label cui_label01 "Custom Icon Maker"
	label cui_label02 "Version 0.55 - 01/31/01"
	label cui_label04 "by Borislav Petrov"
	label cui_label05 "http://gfxcentral.com/bobo/"
)

group "Icon Images"
(
	checkbutton save_i "RGB" width:34 height:30 across:3 align:#left highlightcolor:green checked:true tooltip:"Toggle RGB Image Saving"
	checkbutton save_a "Alpha" width:34 height:30 across:3 align:#center highlightcolor:green checked:false tooltip:"Toggle Alpha Image Saving"
	checkbutton save_m "Mask" width:34 height:30 across:3 align:#right highlightcolor:green checked:false tooltip:"Toggle Maks Image Saving"

	button preview_16i width:34 height:30 across:3 images:#(void_bitmap16, undefined, 1,1,1,1,1) align:#left tooltip:"Preview 16x15 RGB Image"
	button preview_16a width:34 height:30 images:#(void_bitmap16, undefined, 1,1,1,1,1) align:#center tooltip:"Preview 16x15 Alpha Image"
	button preview_16m width:34 height:30 images:#(void_bitmap16, undefined, 1,1,1,1,1) align:#right tooltip:"Preview 16x15 Mask Image"

	button preview_24i width:34 height:30 across:3 images:#(void_bitmap24, undefined, 1,1,1,1,1) align:#left tooltip:"Preview 24x24 RGB Image"
	button preview_24a width:34 height:30 images:#(void_bitmap24, undefined, 1,1,1,1,1) align:#center tooltip:"Preview 24x24 Alpha Image"
	button preview_24m width:34 height:30 images:#(void_bitmap24, undefined, 1,1,1,1,1) align:#right tooltip:"Preview 24x24 Mask Image"
)

group "Preview"
(
	button preview_bmp "" width:136 height:130 align:#center images:#(void_bitmap120, undefined, 1,1,1,1,1) tooltip:"Press to GET current library icon"
)

button make_icon "RENDER ICONS" width:140 height:30 tooltip:"Render Current Viewport" align:#center 
button put_icon "PUT INTO LIBRARY" width:140 height:30 tooltip:"Put Rendering Into Library" align:#center enabled:false

group "Current Library"
(
	spinner icon_number "Icon Number:" range:[1,2,1] type:#integer fieldwidth:40
	dropdownlist load_libraries "" items:#()
	radiobuttons sort_yes labels:#("Last on Top","A..Z,a..z","Creation Order")
	button get_path "Get Library Path" width:100 align:#left across:2 tooltip:"Get a New User Path"
	button get_user "User" width:35 align:#right tooltip:"Recall Last User Path"
	edittext pathname "" fieldwidth:130 align:#left
)

group "Create New Library"
(
	edittext prefix "New Name:" text:"" fieldwidth:75 align:#right 
	button create_library "Create Library" width:140 align:#center 
)

group "Options"
(
	checkbutton verbose_yes "Show System Warnings" width:140 checked:true align:#center 
	checkbutton decal_yes "Mask Upper Left Pixel" width:140 checked:true align:#center 
	checkbutton decal_no "Mask User Color" width:140 checked:false align:#center 
	colorpicker user_mask "User Color"  align:#right color:[0,0,0]
)


--------------------------------------------------------------------
--FUNCTION TO DISPLAY SELECTED LIBRARY FROM THE DATABASE------------
--------------------------------------------------------------------

fn get_library lls = 
(
	if put_icon.enabled then
	(
		close dlib_bmp
		lib_bmp = openbitmap lib_database[lls]
		lib_w = lib_bmp.width
		lib_h = lib_bmp.height
		num_icons = (lib_w/24)+1
		icon_number.range = [1,num_icons,num_icons]
		olib_bmp = bitmap (lib_w+24) lib_h 
		dlib_bmp = bitmap (lib_w+24) lib_h 
		dlib_bmp.filename=("Icon Library: "+load_libraries.items[load_libraries.selection])
		for i = 0 to lib_h do
		(
			pixel_line = getpixels lib_bmp [0,i] lib_w
			setpixels olib_bmp [0,i] pixel_line
		)
		global d_lib = display_icon_number olib_bmp lib_w dlib_bmp icon_number.value
		--display dlib_bmp
		display d_lib
	)
)

--------------------------------------------------------------------
--FUNCTION TO BUILD LIBRARY DATABASE FROM FILES IN A DIRECTORY------
--------------------------------------------------------------------

fn update_library_database =
(
		file_list = getfiles (pathname.text+"\\*.bmp")
		if sort_yes.state == 2 then sort file_list
		if sort_yes.state == 1 then 
		(
			temp_array = #()
			for i in file_list do append temp_array i
			for i = temp_array.count to 1 by -1 do
			(
				file_list[temp_array.count+1-i] = temp_array[i]
			)			
		)
		
		lib_list = #()
		lib_database = #()
		global alpha_database = #()
		global mask_database = #()
		for i = 1 to file_list.count do
		(
			file_name = (filenamefrompath file_list[i]) 
			find_string = findstring file_name "_24i.bmp" 
			if find_string != undefined and find_string>1 then 
			(
				append lib_list (substring file_name 1 (find_string-1))
				append lib_database file_list[i]
			)
		)
		for i = 1 to lib_database.count do
		(
			file_list = getfiles (pathname.text+"\\"+lib_list[i]+"_24a.bmp") 
			if file_list.count == 1 then alpha_database[i] = true else alpha_database[i] = false
			file_list = getfiles (pathname.text+"\\"+lib_list[i]+"_24m.bmp") 
			if file_list.count == 1 then mask_database[i] = true else mask_database[i] = false
		)
		if lib_list.count != 0 then
		(		
			load_libraries.items = lib_list
			put_icon.enabled = true
		)
		else
		(
			put_icon.enabled = false
			load_libraries.items = #()
		)
		
)


--------------------------------------------------------------------
--EVENT HANDLERS----------------------------------------------------
--------------------------------------------------------------------

on sort_yes changed state do
(
	current_lib = lib_list[load_libraries.selection]
	
	update_library_database ()
	get_library load_libraries.selection
	for i = 1 to lib_list.count do
		(
			if current_lib == lib_list[i] then load_libraries.selection = i
		)
	get_library load_libraries.selection
)

on get_path pressed do
(
pi_name = getSavePath caption: "CustomUI Path:"
	if pi_name != undefined then
	(
		pathname.text = (pi_name)
		ini_file = createFile (scriptspath+"\\cimconfig.ini")
		format "%\n" (pathname.text) to:ini_file
		close ini_file
		update_library_database ()
		get_library load_libraries.selection
	)
)

on get_user pressed do
(
path_set = openfile (scriptspath+"\\cimconfig.ini")
		if path_set != undefined then
		(	
			try
			(
				str = readline path_set
				pathname.text = str
				update_library_database ()
				get_library load_libraries.selection			
			)
			catch
			(
				message "No User Path Defined Yet!" title:"CIM Error"
			)
		close path_set	
		)
)


on icon_number changed value do
(
	global d_lib = display_icon_number olib_bmp lib_w dlib_bmp icon_number.value
	display d_lib
)

on load_libraries selected itm do
(
	save_i.checked = true
	if alpha_database[itm] then save_a.checked = true else save_a.checked = false
	if mask_database[itm] then save_m.checked = true else save_m.checked = false
	get_library itm
)

on decal_yes changed state do
(
	decal_no.checked = not state
)

on decal_no changed state do
(
	decal_yes.checked = not state
)


--------------------------------------------------------------------
--GET ICON FROM THE CURRENT LIBRARY --------------------------------
--------------------------------------------------------------------

on preview_bmp pressed do
(
	q = true
	if render_done_flag and verbose_yes.checked then
	(
		txt = "CIM:\nLoad Library Icon into Render Buffer?"
		q = querybox txt title:"CIM ICON LIBRARY"
	)

if q then 
(
	lls = load_libraries.selection
	tmp_bmp = bitmap 24 24
	global out_file16 = bitmap 16 15
	global out_file24 = bitmap 24 24
	global out_file16a = bitmap 16 15 
	global out_file24a = bitmap 24 24 
	global out_file16m = bitmap 16 15 
	global out_file24m = bitmap 24 24
	no_alpha_flag = true
	no_mask_flag = true

	input_file24 = openbitmap (pathname.text+"\\"+lib_list[lls]+"_24i.bmp") 
	input_file16 = openbitmap (pathname.text+"\\"+lib_list[lls]+"_16i.bmp") 
	try
	(
		input_file24a = openbitmap (pathname.text+"\\"+lib_list[lls]+"_24a.bmp") 
		input_file16a = openbitmap (pathname.text+"\\"+lib_list[lls]+"_16a.bmp") 
		no_alpha_flag = false
	)
	catch
	(
		no_alpha_flag = true
	)

	try
	(
		input_file24m = openbitmap (pathname.text+"\\"+lib_list[lls]+"_24m.bmp") 
		input_file16m = openbitmap (pathname.text+"\\"+lib_list[lls]+"_16m.bmp") 
		no_mask_flag = false
	)
	catch
	(
		no_mask_flag = true
	)


	for i = 0 to 23 do
	(
		pixel_line = getpixels input_file24 [(icon_number.value-1)*24,i] 24
		setpixels tmp_bmp [0,i] pixel_line
		setpixels out_file24 [0,i] pixel_line

		if not no_alpha_flag then
		(
			pixel_line = getpixels input_file24a [(icon_number.value-1)*24,i] 24
			setpixels out_file24a [0,i] pixel_line
		)

		if not no_mask_flag then
		(
			pixel_line = getpixels input_file24m [(icon_number.value-1)*24,i] 24
			setpixels out_file24m [0,i] pixel_line
		)

	)--end i loop

	for i = 0 to 14 do
	(
		pixel_line = getpixels input_file16 [(icon_number.value-1)*16,i] 16
		setpixels out_file16 [0,i] pixel_line

		if not no_alpha_flag then
		(
			pixel_line = getpixels input_file16a [(icon_number.value-1)*16,i] 16
			setpixels out_file16a [0,i] pixel_line
		)

		if not no_mask_flag then
		(
			pixel_line = getpixels input_file16m [(icon_number.value-1)*16,i] 16
			setpixels out_file16m [0,i] pixel_line
		)
	)--end i loop

	temp_bmp = bitmap 120 120
	copy tmp_bmp temp_bmp
	preview_bmp.images = #(temp_bmp, undefined, 1,1,1,1,1)

	preview_16i.images = #(out_file16, undefined, 1,1,1,1,1)
	preview_16a.images = #(out_file16a, undefined, 1,1,1,1,1)
	preview_16m.images = #(out_file16m, undefined, 1,1,1,1,1)
	preview_24i.images = #(out_file24, undefined, 1,1,1,1,1)
	preview_24a.images = #(out_file24a, undefined, 1,1,1,1,1)
	preview_24m.images = #(out_file24m, undefined, 1,1,1,1,1)

	render_done_flag = true

)--end if q
)--end on preview_bmp pressed



--------------------------------------------------------------------
--CREATE NEW EMPTY LIBRARY------------------------------------------
--------------------------------------------------------------------

on create_library pressed do
(
	if prefix.text != "" then
	(
		check_for_file = getfiles (pathname.text+"\\"+prefix.text+"_24i.bmp")
		go_on = true
		if check_for_file.count == 1 then
		(
			txt = "Icon Library File\n"+prefix.text+"\nalready exists.\nOverwrite?"
			go_on = querybox txt title:"CIM WARNING"
		)

		if go_on then
		(
			if save_i.checked then
			(
				new_bmp = bitmap 24 24 filename:(pathname.text+"\\"+prefix.text+"_24i.bmp")
				save new_bmp
				close new_bmp
				new_bmp = bitmap 16 15 filename:(pathname.text+"\\"+prefix.text+"_16i.bmp")
				save new_bmp
				close new_bmp
			)
			if save_a.checked then
			(
				new_bmp = bitmap 24 24 filename:(pathname.text+"\\"+prefix.text+"_24a.bmp")
				save new_bmp
				close new_bmp
				new_bmp = bitmap 16 15 filename:(pathname.text+"\\"+prefix.text+"_16a.bmp")
				save new_bmp
				close new_bmp
			)
			if save_m.checked then
			(
				new_bmp = bitmap 24 24 filename:(pathname.text+"\\"+prefix.text+"_24m.bmp")
				save new_bmp
				close new_bmp
				new_bmp = bitmap 16 15 filename:(pathname.text+"\\"+prefix.text+"_16m.bmp")
				save new_bmp
				close new_bmp
			)
		
			update_library_database ()
			for i = 1 to load_libraries.items.count do
			(
				if prefix.text == load_libraries.items[i] then load_libraries.selection = i
			)
			get_library load_libraries.selection
		
		)--end go on
	)
)


--------------------------------------------------------------------
--MAKE ICON --------------------------------------------------------
--------------------------------------------------------------------

on make_icon pressed do
(
	gc ()
	out_file16 = bitmap 16 15
	out_file24 = bitmap 24 24
	
	render to:out_file16 vfb:off
	render to:out_file24 vfb:off

--------------------------------------------------------------------

	out_file16a = bitmap 16 15 
	out_file24a = bitmap 24 24 
	out_file16m = bitmap 16 15 
	out_file24m = bitmap 24 24


	temp_bmp = bitmap 120 120
	copy out_file24 temp_bmp
	preview_bmp.images = #(temp_bmp, undefined, 1,1,1,1,1)

--------------------------------------------------------------------
--EXTRACT ALPHA INFO -----------------------------------------------
--------------------------------------------------------------------

	for i = 0 to 14 do
	(
		alpha_p = #()
		g_p = getpixels out_file16 [0,i] 16
		for j = 1 to 16 do
		(
			alpha_p[j] = [g_p[j].alpha,g_p[j].alpha,g_p[j].alpha] as color
		)
		setpixels out_file16a [0,i] alpha_p
	)--end i loop


	for i = 0 to 23 do
	(
		alpha_p = #()
		g_p = getpixels out_file24 [0,i] 24
		for j = 1 to 24 do
			(
			alpha_p[j] = [g_p[j].alpha,g_p[j].alpha,g_p[j].alpha] as color
			)
		setpixels out_file24a [0,i] alpha_p
	)--end i loop



--------------------------------------------------------------------
--GENERATE MASK FILES-----------------------------------------------
--------------------------------------------------------------------

	bg_color =#()
	if decal_yes.checked then 
	(
		bg_color = getpixels out_file16 [0,0] 1
	)
	else
	(
		bg_color[1] = user_mask.color
	)

	for i = 0 to 14 do
	(
		mask_p = #()
		g_p = getpixels out_file16 [0,i] 16
		for j = 1 to 16 do
		(
			if g_p[j].r as integer == bg_color[1].r and g_p[j].g as integer == bg_color[1].g and g_p[j].b as integer == bg_color[1].b then 
			(
				mask_p[j] = [255,255,255]
			)
			else 
			(
				mask_p[j] = [0,0,0] as color 
			)
		)--end j loop
	setpixels out_file16m [0,i] mask_p
	)--end i loop

	for i = 0 to 23 do
	(
		mask_p = #()
		g_p = getpixels out_file24 [0,i] 24
		for j = 1 to 24 do
		(
			if g_p[j].r as integer == bg_color[1].r and g_p[j].g as integer == bg_color[1].g and g_p[j].b as integer == bg_color[1].b then 
			(
				mask_p[j] = [255,255,255]
			)
			else 
			(
				mask_p[j] = [0,0,0] as color 
			)
		)--end j loop
		setpixels out_file24m [0,i] mask_p
	)

	preview_16i.images = #(out_file16, undefined, 1,1,1,1,1)
	preview_16a.images = #(out_file16a, undefined, 1,1,1,1,1)
	preview_16m.images = #(out_file16m, undefined, 1,1,1,1,1)
	preview_24i.images = #(out_file24, undefined, 1,1,1,1,1)
	preview_24a.images = #(out_file24a, undefined, 1,1,1,1,1)
	preview_24m.images = #(out_file24m, undefined, 1,1,1,1,1)

	render_done_flag = true

)--end on make



--------------------------------------------------------------------
--PUT ICON INTO CURRENT LIBRARY-------------------------------------
--------------------------------------------------------------------

on put_icon pressed do
(
	if render_done_flag then
	(
		lls = load_libraries.selection

		if save_i.checked then
		(
			input_file24 = openbitmap (pathname.text+"\\"+lib_list[lls]+"_24i.bmp") 
			input_file16 = openbitmap (pathname.text+"\\"+lib_list[lls]+"_16i.bmp") 

			if icon_number.value == ((input_file24.width/24)+1) then
			(
				add_new = true
			)
			else
			(
			add_new = false
			)

		q = false
		if not add_new then 
		(
			txt = "Press YES to INSERT between Icons\nPress NO to OVERWRITE Icon\n" 
			q = querybox txt title:"CIM ICON LIBRARY"
		)

		if not q then
		(
			if icon_number.value == ((input_file24.width/24)+1) then
			(
				output_file24 = bitmap (input_file24.width+24) 24
				output_file16 = bitmap (input_file16.width+16) 15
			)
			else
			(
				output_file24 = bitmap (input_file24.width) 24
				output_file16 = bitmap (input_file16.width) 15
			)

			for i = 0 to 23 do
			(
				input_line = getpixels input_file24 [0,i] input_file24.width
				setpixels output_file24 [0,i] input_line
				pixel_line = getpixels out_file24 [0,i] 24
				setpixels olib_bmp [(icon_number.value-1)*24,i] pixel_line
				setpixels output_file24 [(icon_number.value-1)*24,i] pixel_line
			)--end i loop
	
			for i = 0 to 14 do
			(
				input_line = getpixels input_file16 [0,i] input_file16.width
				setpixels output_file16 [0,i] input_line

				pixel_line = getpixels out_file16 [0,i] 16
				setpixels output_file16 [(icon_number.value-1)*16,i] pixel_line
			)
		)	
		else
		(
			output_file24 = bitmap (input_file24.width+24) 24
			output_file16 = bitmap (input_file16.width+16) 15

			for i = 0 to 23 do
			(
				input_line = getpixels input_file24 [0,i] ((icon_number.value-1)*24)
				setpixels output_file24 [0,i] input_line
				input_line = getpixels input_file24 [((icon_number.value-1)*24),i] (input_file24.width-((icon_number.value-1)*24))
				setpixels output_file24 [((icon_number.value)*24),i] input_line
	
				pixel_line = getpixels out_file24 [0,i] 24
				setpixels output_file24 [(icon_number.value-1)*24,i] pixel_line
			)--end i loop
			
			for i = 0 to 14 do
			(
				input_line = getpixels input_file16 [0,i] ((icon_number.value-1)*16)
				setpixels output_file16 [0,i] input_line

				input_line = getpixels input_file16 [((icon_number.value-1)*16),i] (input_file16.width-((icon_number.value-1)*16))
				setpixels output_file16 [(icon_number.value)*16,i] input_line
	
				pixel_line = getpixels out_file16 [0,i] 16
				setpixels output_file16 [(icon_number.value-1)*16,i] pixel_line
			)
		)	
	
	
		close input_file24
		close input_file16
	
		output_file24.filename = (pathname.text+"\\"+lib_list[lls]+"_24i.bmp") 	
		output_file16.filename = (pathname.text+"\\"+lib_list[lls]+"_16i.bmp") 	
	
		save output_file24
		save output_file16

		close output_file24
		close output_file16
	)

	if save_a.checked then
	(
		input_file24 = openbitmap (pathname.text+"\\"+lib_list[lls]+"_24a.bmp") 
		input_file16 = openbitmap (pathname.text+"\\"+lib_list[lls]+"_16a.bmp") 

		if not q then
		(
			if icon_number.value == ((input_file24.width/24)+1) then
			(
				output_file24 = bitmap (input_file24.width+24) 24
				output_file16 = bitmap (input_file16.width+16) 15
			)
			else
			(
				output_file24 = bitmap (input_file24.width) 24
				output_file16 = bitmap (input_file16.width) 15
			)

			for i = 0 to 23 do
			(
				input_line = getpixels input_file24 [0,i] input_file24.width
				setpixels output_file24 [0,i] input_line
				pixel_line = getpixels out_file24a [0,i] 24
				setpixels output_file24 [(icon_number.value-1)*24,i] pixel_line
			)--end i loop
		
			for i = 0 to 14 do
			(
				input_line = getpixels input_file16 [0,i] input_file16.width
				setpixels output_file16 [0,i] input_line
				pixel_line = getpixels out_file16a [0,i] 16
				setpixels output_file16 [(icon_number.value-1)*16,i] pixel_line
			)
		)	
		else
		(
			output_file24 = bitmap (input_file24.width+24) 24
			output_file16 = bitmap (input_file16.width+16) 15

			for i = 0 to 23 do
			(
				input_line = getpixels input_file24 [0,i] ((icon_number.value-1)*24)
				setpixels output_file24 [0,i] input_line
				input_line = getpixels input_file24 [((icon_number.value-1)*24),i] (input_file24.width-((icon_number.value-1)*24))
				setpixels output_file24 [((icon_number.value)*24),i] input_line
	
				pixel_line = getpixels out_file24a [0,i] 24
				setpixels output_file24 [(icon_number.value-1)*24,i] pixel_line
			)--end i loop
		
			for i = 0 to 14 do
			(
				input_line = getpixels input_file16 [0,i] ((icon_number.value-1)*16)
				setpixels output_file16 [0,i] input_line

				input_line = getpixels input_file16 [((icon_number.value-1)*16),i] (input_file16.width-((icon_number.value-1)*16))
				setpixels output_file16 [(icon_number.value)*16,i] input_line
	
				pixel_line = getpixels out_file16a [0,i] 16
				setpixels output_file16 [(icon_number.value-1)*16,i] pixel_line
			)--end i loop
		)	
	
		close input_file24
		close input_file16
	
		output_file24.filename = (pathname.text+"\\"+lib_list[lls]+"_24a.bmp") 	
		output_file16.filename = (pathname.text+"\\"+lib_list[lls]+"_16a.bmp") 	
		save output_file24
		save output_file16
		close output_file24
		close output_file16
	)

	if save_m.checked then
	(
		input_file24 = openbitmap (pathname.text+"\\"+lib_list[lls]+"_24m.bmp") 
		input_file16 = openbitmap (pathname.text+"\\"+lib_list[lls]+"_16m.bmp") 

		if not q then
		(
			if icon_number.value == ((input_file24.width/24)+1) then
			(
				output_file24 = bitmap (input_file24.width+24) 24
				output_file16 = bitmap (input_file16.width+16) 15
			)
			else
			(
				output_file24 = bitmap (input_file24.width) 24
				output_file16 = bitmap (input_file16.width) 15
			)

			for i = 0 to 23 do
			(
				input_line = getpixels input_file24 [0,i] input_file24.width
				setpixels output_file24 [0,i] input_line
				pixel_line = getpixels out_file24m [0,i] 24
				setpixels output_file24 [(icon_number.value-1)*24,i] pixel_line
			)
			for i = 0 to 14 do
			(
				input_line = getpixels input_file16 [0,i] input_file16.width
				setpixels output_file16 [0,i] input_line
				pixel_line = getpixels out_file16m [0,i] 16
				setpixels output_file16 [(icon_number.value-1)*16,i] pixel_line
			)
		)	
		else
		(
			output_file24 = bitmap (input_file24.width+24) 24
			output_file16 = bitmap (input_file16.width+16) 15

			for i = 0 to 23 do
			(
				input_line = getpixels input_file24 [0,i] ((icon_number.value-1)*24)
				setpixels output_file24 [0,i] input_line
				input_line = getpixels input_file24 [((icon_number.value-1)*24),i] (input_file24.width-((icon_number.value-1)*24))
				setpixels output_file24 [((icon_number.value)*24),i] input_line
	
				pixel_line = getpixels out_file24m [0,i] 24
				setpixels output_file24 [(icon_number.value-1)*24,i] pixel_line
			)
			for i = 0 to 14 do
			(
				input_line = getpixels input_file16 [0,i] ((icon_number.value-1)*16)
				setpixels output_file16 [0,i] input_line

				input_line = getpixels input_file16 [((icon_number.value-1)*16),i] (input_file16.width-((icon_number.value-1)*16))
				setpixels output_file16 [(icon_number.value)*16,i] input_line
	
				pixel_line = getpixels out_file16m [0,i] 16
				setpixels output_file16 [(icon_number.value-1)*16,i] pixel_line
			)
		)	
		close input_file24
		close input_file16
	
		output_file24.filename = (pathname.text+"\\"+lib_list[lls]+"_24m.bmp") 	
		output_file16.filename = (pathname.text+"\\"+lib_list[lls]+"_16m.bmp") 	

		save output_file24
		save output_file16
		close output_file24
		close output_file16
	)
	update_library_database ()
	get_library load_libraries.selection
)
else
(
messagebox "Please render an icon first." title:"CIM Message"
)
)--end put icon


--------------------------------------------------------------------
--RENDERED ICON PREVIEWS--------------------------------------------
--------------------------------------------------------------------

on preview_16i pressed do
(
	temp_bmp = bitmap 80 80
	copy out_file16 temp_bmp
	preview_bmp.images = #(temp_bmp, undefined, 1,1,1,1,1)
)

on preview_24i pressed do
(
	temp_bmp = bitmap 120 120
	copy out_file24 temp_bmp
	preview_bmp.images = #(temp_bmp, undefined, 1,1,1,1,1)
)

on preview_16a pressed do
(
	temp_bmp = bitmap 80 80
	copy out_file16a temp_bmp
	preview_bmp.images = #(temp_bmp, undefined, 1,1,1,1,1)
)

on preview_24a pressed do
(
	temp_bmp = bitmap 120 120       
	copy out_file24a temp_bmp
	preview_bmp.images = #(temp_bmp, undefined, 1,1,1,1,1)
)

on preview_16m pressed do
(
	temp_bmp = bitmap 80 80
	copy out_file16m temp_bmp
	preview_bmp.images = #(temp_bmp, undefined, 1,1,1,1,1)
)

on preview_24m pressed do
(
	temp_bmp = bitmap 120 120
	copy out_file24m temp_bmp
	preview_bmp.images = #(temp_bmp, undefined, 1,1,1,1,1)
)


--------------------------------------------------------------------
--OPEN UTILITY------------------------------------------------------
--------------------------------------------------------------------

on custom_icon_maker open do
(
	renderheight=480
	renderwidth=480
	displaysafeframes = true
	max views redraw
	pathname.text = (GetDir #ui)+"\\icons"
	
	update_library_database ()
	get_library load_libraries.selection
	save_i.checked = true
	if alpha_database[load_libraries.selection] then save_a.checked = true else save_a.checked = false
	if mask_database[load_libraries.selection] then save_m.checked = true else save_m.checked = false
	render_done_flag = false
)

--------------------------------------------------------------------
--CLOSE UTILITY-----------------------------------------------------
--------------------------------------------------------------------

on custom_icon_maker close do
(
	close d_lib
)

)--end utility